package com.qmth.distributed.print.business.service;

import com.qmth.distributed.print.business.bean.auth.AuthBean;
import com.qmth.distributed.print.business.bean.dto.PrivilegeCacheDto;
import com.qmth.distributed.print.business.bean.result.RolePrivilegeResult;
import com.qmth.distributed.print.business.entity.SysRolePrivilege;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.entity.SysUserRole;
import com.qmth.distributed.print.business.enums.PrivilegePropertyEnum;

import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchAlgorithmException;
import java.util.List;

/**
 * @Description: 公共服务service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/26
 */
public interface CommonService {

    /**
     * 新增用户权限
     *
     * @param sysUser
     * @param roleId
     */
    public void addUserRolePrivilege(SysUser sysUser, Long roleId);

    /**
     * 获取用户角色
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> getUserRolePrivilege(Long userId);

    /**
     * 获取角色权限
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> getRolePrivilege(Long roleId);

    /**
     * 获取用户权限
     *
     * @param userId
     * @return
     */
    public AuthBean getUserAuth(Long userId);

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public List<String> getPrivilegeUrl(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 删除用户信息
     *
     * @param userId
     */
    public void removeUserInfo(Long userId) throws NoSuchAlgorithmException;

    /**
     * 修改角色状态
     *
     * @param roleIds
     * @param enable
     */
    public void updateRoleEnable(Long[] roleIds, Boolean enable);

    /**
     * 查询用户或角色权限信息
     *
     * @param privilegeCacheDtoList
     * @return
     */
    public RolePrivilegeResult getUserOrRolePrivilege(List<PrivilegeCacheDto> privilegeCacheDtoList);

    /**
     * 文件预览
     *
     * @param path
     * @return
     */
    public String filePreview(String path);

    /**
     * 题卡html文件上传，读取文件内容
     *
     * @param path
     * @return
     */
    public String readFileContent(String path);

    /**
     * 获取文件流
     *
     * @param path
     * @param pdf
     * @return
     */
    public InputStream getFileInputStream(String path, boolean pdf) throws IOException;

    /**
     * 获取文件
     *
     * @param path
     * @param pdf
     * @return
     */
    public String getFile(String path, boolean pdf) throws IOException;
}
