package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.excel.UserImportDto;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.GoogleBarCodeUtil;
import com.qmth.distributed.print.business.util.OssUtil;
import com.qmth.distributed.print.business.util.PdfUtil;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.Base64Util;
import com.qmth.distributed.print.common.util.HexUtils;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    CommonService commonService;

    @Resource
    ExamDetailService detailService;

    @Resource
    ExamDetailCourseService detailCourseService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamCardService examCardService;

    @Resource
    ExamCardDetailService examCardDetailService;

    @Resource
    ExamStudentService examStudentService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    OssUtil ossUtil;

    /**
     * 执行导入用户逻辑
     *
     * @param finalList
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Gson gson = new Gson();
        LinkedMultiValueMap<Long, SysUser> tbUserLinkedMultiValueMap = new LinkedMultiValueMap<>();
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> userImportDtoList = excelMap.get(i);
            for (int y = 0; y < userImportDtoList.size(); y++) {
                UserImportDto userImportDto = (UserImportDto) userImportDtoList.get(y);
                SysUser sysUser = gson.fromJson(gson.toJson(userImportDto), SysUser.class);

                //查用户登录名是否重复
                QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
                sysUserQueryWrapper.lambda().eq(SysUser::getLoginName, userImportDto.getLoginName());
                long count = sysUserService.count(sysUserQueryWrapper);
                if (count > 0) {
                    throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[登录名]重复")));
                }
                //查学校是否存在
                if (Objects.nonNull(userImportDto.getSchoolCode())) {
                    QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
                    basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, userImportDto.getSchoolCode());
                    BasicSchool basicSchool = basicSchoolService.getOne(basicSchoolQueryWrapper);
                    if (Objects.isNull(basicSchool)) {
                        throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[学校代码]不存在")));
                    }
                    sysUser.setSchoolId(basicSchool.getId());
                }
                sysUser.setInsertInfo(requestUser.getId());
                sysUser.setPassword(Base64Util.encode(userImportDto.getPassword().getBytes()));
                tbUserLinkedMultiValueMap.add(userImportDto.getSysRole().getId(), sysUser);
            }
        }
        tbUserLinkedMultiValueMap.forEach((k, v) -> {
            sysUserService.saveBatch(v);
            v.forEach(s -> {
                commonService.addUserRolePrivilege(s, k);
            });
        });
        return map;
    }

    /**
     * 创建pdf逻辑
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeCreatePdfLogic(Map<String, Object> map) throws IOException {
        List<BasicAttachment> basicAttachmentList = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
            //这里为保存附件html的逻辑
            //查询题卡详情是否有附件id，有的话则把以前的附件删除
            Set<Long> attachmentIds = null;

            //查询printPlan
            ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
            if (Objects.isNull(examPrintPlan)) {
                throw ExceptionResultEnum.ERROR.exception("印刷计划为空");
            }

            //查询examDetail
            QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
            examDetailQueryWrapper.lambda().eq(ExamDetail::getSchoolId, sysUser.getSchoolId())
                    .eq(ExamDetail::getPrintPlanId, tbTask.getPrintPlanId());
            ExamDetail examDetail = detailService.getOne(examDetailQueryWrapper);
            if (Objects.isNull(examDetail)) {
                throw ExceptionResultEnum.ERROR.exception("考务计划为空");
            }
            attachmentIds = Objects.isNull(attachmentIds) ? attachmentIds = new HashSet<>() : attachmentIds;
            if (Objects.nonNull(examDetail.getAttachmentId())) {
                attachmentIds.add(examDetail.getAttachmentId());
            }

            //查询examDetailCourse
            QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
            examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetail.getId());
            List<ExamDetailCourse> examDetailCourseList = detailCourseService.list(examDetailCourseQueryWrapper);

            StringJoiner stringJoiner = new StringJoiner("").add(SystemConstant.PDF_TEMP_FILES_DIR).add(File.separator);
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                List<String> paperPdfList = new ArrayList<>();
                List<String> cardPdfList = new ArrayList<>();
                List<String> studentPdfList = new ArrayList<>();
                //查询试卷
                QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
                examTaskQueryWrapper.lambda().eq(ExamTask::getSchoolId, sysUser.getSchoolId())
                        .eq(ExamTask::getCourseCode, examDetailCourse.getCourseCode())
                        .eq(ExamTask::getCourseName, examDetailCourse.getCourseName())
                        .eq(ExamTask::getPaperNumber, examDetailCourse.getPaperNumber())
                        .eq(ExamTask::getEnable, true)
                        .eq(ExamTask::getStatus, ExamStatusEnum.FINISH);
                List<ExamTask> examTaskList = examTaskService.list(examTaskQueryWrapper);
                if (Objects.isNull(examTaskList) || examTaskList.size() == 0) {
                    throw ExceptionResultEnum.ERROR.exception("命题任务为空");
                }
                Set<Long> examTaskIds = examTaskList.stream().map(s -> s.getId()).collect(Collectors.toSet());

                QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
                examTaskDetailQueryWrapper.lambda().in(ExamTaskDetail::getExamTaskId, examTaskIds)
                        .eq(ExamTaskDetail::getEnable, true);
                List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.list(examTaskDetailQueryWrapper);
                JSONObject jsonObject = new JSONObject();
                JSONArray jsonArray = new JSONArray();
                for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
                    //查询题卡
                    ExamCard examCard = examCardService.getById(examTaskDetail.getCardId());
                    if (Objects.isNull(examCard)) {
                        throw ExceptionResultEnum.ERROR.exception("题卡为空");
                    }
                    QueryWrapper<ExamCardDetail> examCardDetailQueryWrapper = new QueryWrapper<>();
                    examCardDetailQueryWrapper.lambda().eq(ExamCardDetail::getCardId, examCard.getId());
                    List<ExamCardDetail> examCardDetailList = examCardDetailService.list(examCardDetailQueryWrapper);

                    String[] paperTypes = examTaskDetail.getPaperType().split("/");
                    JSONObject jsonObjectPaper = JSONObject.parseObject(examTaskDetail.getPaperAttachmentIds());
                    JSONArray jsonArrayPaper = jsonObjectPaper.getJSONArray("paper");
                    for (int i = 0; i < jsonArrayPaper.size(); i++) {
                        JSONObject object = (JSONObject) jsonArrayPaper.get(i);
                        if (Objects.nonNull(object.get("attachmentId"))) {
                            Long attachmentId = Long.parseLong(String.valueOf(object.get("attachmentId")));
                            BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                            paperPdfList.add(commonService.getFile(basicAttachment.getPath(), false));
                        }
                    }
                    basicAttachmentList = Objects.isNull(basicAttachmentList) ? basicAttachmentList = new ArrayList<>() : basicAttachmentList;
                    for (ExamCardDetail examCardDetail : examCardDetailList) {
                        if (Objects.nonNull(examCardDetail.getAttachmentId())) {
                            JSONObject jsonObjectCard = JSONObject.parseObject(examCardDetail.getAttachmentId());
                            JSONArray jsonArrayCard = (JSONArray) jsonObjectCard.get("card");
                            for (int i = 0; i < jsonArrayCard.size(); i++) {
                                JSONObject object = (JSONObject) jsonArrayCard.get(i);
                                if (Objects.nonNull(object.get("attachmentId"))) {
                                    attachmentIds.add(Long.parseLong(String.valueOf(object.get("attachmentId"))));
                                }
                            }
                        }

                        //把模板页面上的 ${} 替换成实际内容
                        String content = examCardDetail.getHtmlContent();
                        log.info("contentTemp:{}", content);
                        String cardContent = content;
                        String studentContent = content;
                        cardContent = cardContent.replaceAll("<img src=\"data:image/png;base64,\\$\\{examNumber\\}\">", "");
                        cardContent = cardContent.replaceAll("\\$\\{examNumberStr\\}", "");

                        for (String paperType : paperTypes) {
                            for (Integer integer = 1; integer <= examPrintPlan.getBackupCount(); integer++) {
                                //通用题卡
                                String cardTemp = cardContent;
                                cardTemp = cardTemp.replaceAll("\\$\\{paperTypeName\\}", paperType);
                                //随机生成试卷条码并将图片转成base64
                                cardTemp = cardTemp.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false));
                                //通用题卡生成卷袋贴条码
                                String paperCode = examDetailCourse.getPaperNumber();
                                String paperCodeImg = GoogleBarCodeUtil.createBarCode(paperCode, false);
                                String paperCodeDiv = "<div class=\"page-box page-box-0\"><div class=\"package-number\" style=\"position: absolute;width: 200px;height: 40px;top: 80px;right: 35px;transform: rotate(-90deg);transform-origin: center right;text-align: center;z-index: 99;\"><img src=\"data:image/png;base64," + paperCodeImg + "\" style=\"display: block; height: 28px; width: 100%\" /><p style=\"line-height: 1; font-size: 12px; margin: 0;\">" + paperCode + "</p></div>";
                                cardTemp = cardTemp.replaceAll("<div class=\"page-box page-box-0\">", paperCodeDiv);
                                BasicAttachment cardAttachment = basicAttachmentService.saveAttachmentHtml(examCard.getSchoolId() + "|" + examCard.getCourseCode(), cardTemp, sysUser.getId(), cardPdfList);
                                JSONObject object = new JSONObject();
                                object.put("name", paperType);
                                object.put("examTaskDetailId", examTaskDetail.getId());
                                object.put("attachmentId", cardAttachment.getId());
                                jsonArray.add(object);
                                basicAttachmentList.add(cardAttachment);
                            }

                            //查询考生
                            QueryWrapper<ExamStudent> examStudentQueryWrapper = new QueryWrapper<>();
                            examStudentQueryWrapper.lambda().eq(ExamStudent::getSchoolId, sysUser.getSchoolId())
                                    .eq(ExamStudent::getExamDetailCourseId, examTaskDetail.getId());
                            List<ExamStudent> examStudentList = examStudentService.list(examStudentQueryWrapper);

                            for (ExamStudent t : examStudentList) {
                                if (Objects.nonNull(t.getAttachmentId())) {
                                    attachmentIds.add(t.getAttachmentId());
                                }
                                String studentHtml = studentContent;
                                if (Objects.nonNull(t.getExtendFields())) {
                                    JSONArray jsonObjectExtend = (JSONArray) JSONArray.parse(t.getExtendFields());//扩展字段
                                    if (Objects.nonNull(jsonObjectExtend)) {
                                        for (int i = 0; i < jsonObjectExtend.size(); i++) {
                                            JSONObject object = (JSONObject) jsonObjectExtend.get(i);
                                            studentHtml = studentHtml.replaceAll("\\$\\{" + object.get("code") + "\\}", String.valueOf(object.get("value")));
                                        }
                                    }
                                }
                                //生成学生考号条码并将图片转成base64
                                studentHtml = studentHtml.replaceAll("\\$\\{examNumber\\}", GoogleBarCodeUtil.createBarCode(t.getTicketNumber(), false));
                                studentHtml = studentHtml.replaceAll("\\$\\{examNumberStr\\}", t.getTicketNumber());
                                //随机生成学生试卷条码并将图片转成base64
                                studentHtml = studentHtml.replaceAll("\\$\\{paperType\\}", GoogleBarCodeUtil.createBarCode(SystemConstant.convertPaperType(paperType), false));
                                studentHtml = studentHtml.replaceAll("\\$\\{paperTypeName\\}", paperType);
                                studentHtml = studentHtml.replaceAll("\\$\\{studentCode\\}", t.getStudentCode());
                                studentHtml = studentHtml.replaceAll("\\$\\{name\\}", t.getStudentName());
                                studentHtml = studentHtml.replaceAll("\\$\\{courseName\\}", examCard.getCourseName());

                                //学生题卡
                                BasicAttachment examStudentAttachment = basicAttachmentService.saveAttachmentHtml(examCard.getSchoolId() + "|" + examCard.getCourseCode() + "|" + t.getTicketNumber(), studentHtml, sysUser.getId(), studentPdfList);
                                t.setAttachmentId(examStudentAttachment.getId());
                                basicAttachmentList.add(examStudentAttachment);
                            }
                            examStudentService.saveOrUpdateBatch(examStudentList);
                        }
                        jsonObject.put("card", jsonArray);
                        examCardDetail.setAttachmentId(jsonObject.toJSONString());
                    }
                    examCardDetailService.saveOrUpdateBatch(examCardDetailList);
                    List<String> mergePdfList = new ArrayList<>();
                    mergePdfList.addAll(paperPdfList);
                    mergePdfList.addAll(cardPdfList);
                    mergePdfList.addAll(studentPdfList);
                    String dirName = PdfUtil.mergePdf(mergePdfList.toArray(new String[mergePdfList.size()]), null);
                    File localPdfFile = new File(stringJoiner.toString() + File.separator + dirName);
                    JSONObject jsonPdf = new JSONObject();
                    jsonPdf.put(SystemConstant.TYPE, SystemConstant.OSS);
                    jsonPdf.put(SystemConstant.PATH, dirName);
                    jsonPdf.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.PDF);
                    BasicAttachment basicAttachment = new BasicAttachment(jsonPdf.toJSONString(), localPdfFile.getName(), SystemConstant.PDF_PREFIX, new BigDecimal(localPdfFile.length()), DigestUtils.md5Hex(new FileInputStream(localPdfFile)), sysUser.getId());
                    basicAttachmentService.save(basicAttachment);
                    examDetail.setAttachmentId(basicAttachment.getId());
                    detailService.saveOrUpdate(examDetail);
                    basicAttachmentList.add(basicAttachment);
                    ossUtil.ossUpload(dirName, localPdfFile, BinaryUtil.toBase64String(HexUtils.decodeHex(DigestUtils.md5Hex(new FileInputStream(localPdfFile)))));
                    localPdfFile.delete();
                    for (String s : mergePdfList) {
                        new File(s).delete();
                    }
                }
            }
            //最后一步删除附件
            if (Objects.nonNull(attachmentIds) && attachmentIds.size() > 0) {
                QueryWrapper<BasicAttachment> basicAttachmentQueryWrapper = new QueryWrapper<>();
                basicAttachmentQueryWrapper.lambda().in(BasicAttachment::getId, attachmentIds);
                basicAttachmentService.batchDeleteAttachment(basicAttachmentService.list(basicAttachmentQueryWrapper));
                basicAttachmentService.removeByIds(attachmentIds);
            }
        } catch (Exception e) {
            log.error("请求出错", e);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentList);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }
}
